@@ -52,9 +52,9 @@ class XMLObject(object):
     _name_ = 'root'
     _namespace_ = None
     _member_inherit_namespace_ = True
-    _member_inherit_attributes_ = False
+    _member_inherit_members_ = False
     
-    def __init__(self, name=_name_, namespace=_namespace_, attributes=None, etree=None, parent_etree=None):
+    def __init__(self, name=_name_, namespace=_namespace_, members=None, etree=None, parent_etree=None, attributes=None):
         """Initialization, handles creation of etree object if none exists"""
         if name.find('}') is not -1:
             namespace, name = get_namespace_and_name(name)
@@ -66,12 +66,16 @@ class XMLObject(object):
         else:
             object.__setattr__(self, '_etree_', ElementTree.Element(make_tag(name, namespace)))
         
-        if attributes is not None:
-            getattr(self, '_etree_').attrib = copy.copy(attributes)
+        if members is not None:
+            getattr(self, '_etree_').attrib = copy.copy(members)
         
         object.__setattr__(self, '_name_', name)
         object.__setattr__(self, '_namespace_',  namespace)
         
+        if attributes is not None:
+            for key, value in attributes.items():
+                self[key] = value
+        
     def _set_namespace_(self, namespace):
         object.__setattr__(self, '_namespace_', namespace)
         self._etree_.tag = make_tag(self._name_, self._namespace_)
@@ -80,7 +84,7 @@ class XMLObject(object):
         object.__setattr__(self, '_name_', tag)
         self._etree_.tag = make_tag(self._name_, self._namespace_)
         
-    def _create_new_member_(self, key, value, attributes=None):
+    def _create_new_member_(self, key, value, members=None):
         # Create new member and handle adding xml sub elements
         if value.__class__.__name__ in self._text_value_conversions_:
             if self._member_inherit_namespace_ is True and self._namespace_ is not None:
@@ -88,11 +92,11 @@ class XMLObject(object):
             else:
                 element = ElementTree.SubElement(self._etree_, key)
             
-            if self._member_inherit_attributes_ is True and len(self._etree_.items()) is not 0:
+            if self._member_inherit_members_ is True and len(self._etree_.items()) is not 0:
                 element.attrib = copy.copy(self._etree_.attrib)
             
-            if attributes is not None:
-                element.attrib.update(attributes)
+            if members is not None:
+                element.attrib.update(members)
         
             # if type(value) is list:
             #     new_member = self._add_xobject_member_(key, XMLObject())
@@ -111,27 +115,50 @@ class XMLObject(object):
             else:
                 namespace = None
             
-            inherit_attributes = None    
+            inherit_members = None    
                 
-            if self._member_inherit_attributes_ is True and len(self._etree_.attrib) is not 0:
-                inherit_attributes = copy.copy(self._etree_.attrib)
+            if self._member_inherit_members_ is True and len(self._etree_.attrib) is not 0:
+                inherit_members = copy.copy(self._etree_.attrib)
             
-            if attributes is not None:
-                attributes = inherit_attributes.update(attributes)
-            elif inherit_attributes:
-                attributes = inherit_attributes
+            if members is not None:
+                members = inherit_members.update(members)
+            elif inherit_members:
+                members = inherit_members
                 
-            object.__setattr__(self, key, XMLObject(name=key, namespace=namespace, attributes=attributes, parent_etree=self._etree_))
+            object.__setattr__(self, key, XMLObject(name=key, namespace=namespace, members=members, parent_etree=self._etree_))
         else:
             raise TypeError, "Unsupported object type"
             
-        
+    def __getattr__(self, key):
+        """Get attribute by name. Supports '{namespace}key' regardless of attribute type."""
+        if key.find('}') is not -1:
+            real_key = key.split('}')
+            real_key[0] = real_key[0].replace('{', '')
+            
+            try:
+                attribute_by_name = object.__getattribute__(self, real_key[1])
+            except KeyError, e:
+                raise AttributeError, e
+                
+            if type(attribute_by_name) is element_list:
+                for element in attribute_by_name:
+                    if element._namespace_ == real_key[0]:
+                        return element
+                raise AttributeError, 'no element with namespace %s exists in %s' % (real_key[0], attribute_by_name)
+            else:
+                if attribute_by_name._namespace_ == real_key[0]:
+                    return attribute_by_name
+                else:
+                    raise AttributeError, '%s does not match namespace %s' % (attribute_by_name, real_key[1])
+        else:
+            raise AttributeError, 'object does not have attributes %s' % key
+            
         
     def __setattr__(self, key, value):
         """Attribute assignment method that handles xml manipulation. Values are assumed to be xml child nodes"""
         if key.startswith('_') is False:
             if hasattr(self, key):
-                # if that attribute exists modify the value properly and then create a new xobject type and assign it to he object
+                # if that member exists modify the value properly and then create a new xobject type and assign it to he object
                 # Note: this doesn't handle the case of when the type is XMLObject yet.
                 if value.__class__.__name__ in self._text_value_conversions_:
                     current_value = getattr(self, key)
@@ -164,12 +191,79 @@ class XMLObject(object):
         object.__delattr__(self, key)
             
     def __setitem__(self, key, value):
-        """Item assignment method that handles xml manipulation. Values are assumed to be xml attributes"""
+        """Item assignment method that handles xml manipulation. Values are assumed to be xml members"""
         self._etree_.attrib.__setitem__(key, self._text_value_conversions_[value.__class__.__name__](value))
         
     def __getitem__(self, key):
         """Item get method that interfaces with etree"""
         return self._etree_.attrib.__getitem__(key)
+        
+    
+    ### Object comparisons ###    
+    def _comparison_(self, other):
+        """xmlobject comparison method"""
+        
+        def compare_members(object_one, object_two, member_name):
+            """method for comparing a single member on two objects by name"""
+            if hasattr(object_one, member_name):
+                if not hasattr(object_two, member_name):
+                    return False
+                elif getattr(object_one, member_name) != getattr(object_two, member_name):
+                    return False
+                elif not getattr(object_one, member_name) == getattr(object_two, member_name):
+                    return False
+            return True
+        
+        # Comparisons for members
+        for member in [member for member in dir(self) if not member.startswith('_')]:
+            if not compare_members(self, other, member):
+                return False
+                
+                    
+            # Attribute comparison for members
+            if hasattr(getattr(self, member), '_etree_'):
+                if not hasattr(getattr(other, member), '_etree_'):
+                    return False
+                else:
+                    if getattr(self, member)._etree_.attrib != getattr(other, member)._etree_.attrib:
+                        return False
+            
+            # Name comparison for members (this is a corner case since name should always be the member name)
+            if not compare_members(getattr(self, member), getattr(other, member), '_name_'):
+                return False
+            
+            # Namespace comparison for members
+            if not compare_members(getattr(self, member), getattr(other, member), '_namespace_'):
+                return False
+            
+        # Attribute comparison for root
+        if not hasattr(other, '_etree_'):
+            return False
+        else:
+            if self._etree_.attrib != other._etree_.attrib:
+                return False
+        
+        # Name comparison for root
+        if not compare_members(self, other, '_name_'):
+            return False
+        
+        # Namespace comparison for root
+        if not compare_members(self, other, '_namespace_'):
+            return False
+        
+        return True
+            
+    def __eq__(self, other):
+        if self._comparison_(other):
+            return True
+        else:
+            return False
+    
+    def __ne__(self, other):
+        if not self._comparison_(other):
+            return True
+        else:
+            return False
                 
 
 def test():
